/*************************************************************************
 * asset-storage.h
 *
 * Matt Shelton	<matt@mattshelton.com>
 *
 * This header file contains information relating to the asset-storage.c
 * module.
 *
 * Copyright (C) 2004 Matt Shelton <matt@mattshelton.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: asset-storage.h,v 1.14 2004/09/30 22:25:50 mattshelton Exp $
 *
 **************************************************************************/

/* INCLUDES ---------------------------------------- */
#include <unistd.h>
#include <stdio.h>
#include <signal.h>
#include <time.h>

#ifdef BSD
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in_systm.h>
#include <sys/socket.h>
#endif

#ifdef FREEBSD
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in_systm.h>
#include <sys/socket.h>
#endif

#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include "global.h"


/* DEFINES ----------------------------------------- */
#ifndef __FAVOR_BSD
#define __FAVOR_BSD
#endif
#ifndef _BSD_SOURCE
#define _BSD_SOURCE
#endif


/* PROTOTYPES -------------------------------------- */
int check_tcp_asset (struct in_addr ip_addr, u_int16_t port);
int check_icmp_asset (struct in_addr ip_addr);
int check_arp_asset (struct in_addr ip_addr, struct ether_addr mac_addr);
void add_asset (struct in_addr ip_addr, u_int16_t port, unsigned short proto, char service[MAX_SERVICE], char application[MAX_LENGTH], time_t discovered);
void add_arp_asset (struct in_addr ip_addr, struct ether_addr mac_addr, time_t discovered);
void init_output_file (char outfile[255]);
void close_output_file (void);
void read_report_file(char infile[255]);
short print_asset (struct in_addr ip_addr, u_int16_t port, unsigned short proto);
short print_arp_asset (struct in_addr ip_addr, struct ether_addr mac_addr);
unsigned short get_i_attempts (struct in_addr ip_addr, u_int16_t port, unsigned short proto);
short update_i_attempts (struct in_addr ip_addr, u_int16_t port, unsigned short proto, unsigned short i_attempts);
short update_asset (struct in_addr ip_addr, u_int16_t port, unsigned short proto, char *service, char *application);
void free_asset (void);

#ifdef DEBUG
void print_database ();
#endif /* DEBUG */


/* GLOBALS ----------------------------------------- */
