/*************************************************************************
 * global.h
 *
 * Matt Shelton	<matt@mattshelton.com>
 *
 * This header file contains the global values for all the PADS modules
 * and components.
 *
 * Copyright (C) 2004 Matt Shelton <matt@mattshelton.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: global.h,v 1.17 2004/09/27 23:23:08 mattshelton Exp $
 *
 **************************************************************************/

/* DEFINES ------------------------------------------ */
#define MAX_ASSETS 50000
#define MAX_LENGTH 200
#define MAX_SIGNATURES 1000
#define MAX_SERVICE 10
#define MAX_APP 100
#define MAX_VER 25
#define MAX_MISC 100

#define MAX_FILE 255
#define VENDOR_LEN 256
#define MAC_LEN 10

#define I_ATTEMPTS 4

/* #define DEBUG */

#define PADS_SIGNATURE_LIST "pads-signature-list"
#define PADS_ETHER_CODES "pads-ether-codes"

#ifdef BSD
#define MAC_ADDR(x) x.ether_addr_octet
#endif
#ifdef FREEBSD
#define MAC_ADDR(x) x.octet
#endif
#ifdef LINUX
#define MAC_ADDR(x) x.ether_addr_octet
#endif
#ifdef SOLARIS
#define MAC_ADDR(x) x.ether_addr_octet
#endif

#ifdef BSD
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in_systm.h>
#include <sys/socket.h>
#endif

#ifdef FREEBSD
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in_systm.h>
#include <sys/socket.h>
#endif

#include <net/if_arp.h>
#include <netinet/if_ether.h>
#include "pcre-4.5/pcre.h"

/* DATA STRUCTURES ---------------------------------- */
struct asset_storage
{
    struct in_addr ip_addr;		/* Asset IP Address */
    u_int16_t port;			/* Asset Port */
    unsigned short proto;		/* Asset Protocol */
    char service[MAX_SERVICE];		/* Asset Service (i.e. SSH, WWW, etc.) */
    char application[MAX_LENGTH];	/* Asset Application (i.e. Apache, etc.) */
    /* char os [MAX_LENGTH]; */
    time_t discovered;			/* Time at which asset was first seen. */
    unsigned short i_attempts;		/* Attempts at identifying the asset. */
    struct asset_storage *next;		/* Next Signature Structure */
};

/* arp_storage:  Data structure used to store data collected from ARP packets. */
struct arp_storage
{
    struct in_addr ip_addr;		/* Asset IP Address */
    struct ether_addr mac_addr;		/* Asset MAC Address */
    time_t discovered;			/* Time at which asset was first seen. */
    struct arp_storage *next;		/* Next ARP Structure */
};

struct asset_signature
{
    char service[MAX_SERVICE];		/* Service (i.e. SSH, WWW, etc.) */
    struct {				/* Application Title, broken up into 3 parts. */
	char app[MAX_APP];
	char ver[MAX_VER];
	char misc[MAX_MISC];
    } title;
    pcre *regex;			/* Signature - Compiled Regular Expression */
    pcre_extra *study;			/* Studied version of the compiled regex. */
    struct asset_signature *next;	/* Next Signature Structure */
};

/* vendor:  Data structure used to store MAC address to vendor mappings. */
struct vendor {
    unsigned int mac;
    char vendor[VENDOR_LEN];
    struct vendor *next;
};


/* TYPEDEFS ---------------------------------------- */
typedef struct vendor VENDOR;
typedef VENDOR *VENDOR_LINK;

