/*************************************************************************
 *
 * pads-mac.c
 *
 * Dominique Karg <dk at ossim.net>
 * Matt Shelton <matt at mattshelton.com>
 *
 * This module will take MAC addresses from the libpcap stack and attempt
 * to translate them into a vendor code.  Thanks go to Dominique for
 * coding this module!
 *
 * Copyright (C) 2004 Matt Shelton <matt at mattshelton.com>
 * Copyright (C) 2004 Dominique Karg <dk at ossim.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: pads-mac.c,v 1.4 2004/09/29 05:48:43 mattshelton Exp $
 *
 **************************************************************************/
#ifndef DISABLE_VENDOR

#include "pads-mac.h"

VENDOR_LINK vendors = NULL;

void init_vendor () {
    char filename[MAX_FILE];
    FILE *fp;
    char line[80];
    char vendor[80];
    char mac[4];

    int m1, m2, m3, i;

    /* Check for a PADS_ETHER_CODES file within the current directory.  */
    if ((fp = fopen(PADS_ETHER_CODES, "r")) != NULL) {
	snprintf(filename, sizeof(filename), "./%s", PADS_ETHER_CODES);
	fclose(fp);
    } else {
	snprintf(filename, sizeof(filename), "%s/%s", DATADIR, PADS_ETHER_CODES);
    }

    /* Open File */
    fp = fopen(filename,"r");
    if(fp == NULL){
	fprintf(stderr, "[!] ERROR:  Unable to open %s.\n", filename);
	exit(0);
    }

    /* Parse File */
    while(fgets(line, 80, fp) != 0){
	if (sscanf(line, "%02X:%02X:%02X %80[^,\n],\n", &m1, &m2, &m3, vendor) != 4)
	    continue;

	mac[0] = (char) (m1);
	mac[1] = (char) (m2);
	mac[2] = (char) (m3);
	mac[3] = 0;

	vendors = add_vendor (mac, vendor, vendors);
    }
}

VENDOR_LINK add_vendor (char *mac, char *vendor_str, VENDOR_LINK vendors){
    VENDOR_LINK tmp = NULL;
    VENDOR_LINK data = NULL;

    /* Assign data to temporary data structure. */
    tmp = (VENDOR_LINK) malloc (sizeof (VENDOR));
    tmp->mac = *(int *) mac;
    memcpy(tmp->vendor, vendor_str, VENDOR_LEN);
    tmp->vendor[VENDOR_LEN-1] = '\0';
    tmp->next = NULL;

    if(vendors == NULL) {
	vendors = tmp;
    } else {
	data = vendors;
	while (data != NULL) {
	    if (data->next == NULL) {
		data->next = tmp;
		break;
	    } else {
		data = data->next;
	    }
	}
    }

    return (vendors);
}

char *
get_vendor (VENDOR_LINK vendors, char *m)
{
    char mac[4];
    VENDOR_LINK curr;
    curr = vendors;

    /* Prepare MAC for matching. */
    mac[0] = *m++;
    mac[1] = *m++;
    mac[2] = *m;
    mac[3] = 0;

    /* Cycle through data structure looking for match. */
    while (curr != NULL) {
	if (curr->mac == *(u_int *) mac)
	    return curr->vendor;

	curr = curr->next;
    }

    /* Nothing Found */
    return NULL;
}

void free_vendor (VENDOR_LINK vendors){
    VENDOR_LINK curr, next;

    curr = vendors;

    while (curr != NULL) {
	next = curr->next;
	free (curr);
	curr = next;
    }
}

#ifdef DEBUG
void show_vendor (VENDOR_LINK vendors){
    VENDOR_LINK curr;

    curr = vendors;

    while(curr != NULL) {
	printf("Mac: %s\nVendor: %s\n\n", curr->mac, curr->vendor);
	curr = curr->next;
    }

}
#endif /* DEBUG */
#endif /* DISABLE_VENDOR */
