/*************************************************************************
 *
 * util.c
 *
 * Matt Shelton <matt@mattshelton.com>
 *
 * This module contains functions miscellaneous utility functions.
 *
 * Copyright (C) 2004 Matt Shelton <matt@mattshelton.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: util.c,v 1.6 2004/08/26 03:20:09 mattshelton Exp $
 *
 **************************************************************************/
#include "util.h"


/* ----------------------------------------------------------
 * FUNCTION	: strip_comment
 * DESCRIPTION	: This function will strip out anything after
 *		: a '#' within a field.
 * INPUT	: 0 - String
 * RETURN	: 1 - String
 * ---------------------------------------------------------- */
void strip_comment (char *string)
{
    char *pos;

    while ((pos = strchr(string, '#')) != NULL) {
	while (*pos != '\r' && *pos != '\n' && *pos != '\0') {
	    *pos++ = ' ';
	}
    }
}

/* ----------------------------------------------------------
 * FUNCTION	: chomp
 * DESCRIPTION	: This function is similar to Perl's 'chomp'
 *		: command.  It will strip off the '\n'
 *		: closest to the end of the string.
 * INPUT	: 0 - String
 *		: 1 - Size of String
 * RETURN	: 0 - '\n' removed
 *		: 1 - '\n' was never there
 * ---------------------------------------------------------- */
int chomp (char *string, int size)
{
    for (size; size >= 0; size--) {
	if (string[size] == '\n') {
	    string[size] = '\0';
	    return 1;
	}
    }

    return 0;
}

/* ----------------------------------------------------------
 * FUNCTION	: daemonize
 * DESCRIPTION	: This function will place the application in
 *		: the background.
 * INPUT	: None!
 * RETURN	: None!
 * ---------------------------------------------------------- */
void daemonize ()
{
    pid_t pid;

    printf("[-] Daemonizing...\n");

    pid = fork();
    if (pid > 0) {
	/* Parent */
	exit(0);
    } else if (pid < 0) {
	/* Error */
	perror("[!] ERROR:  fork\n");
	exit(0);
    } else {
	/* Child */
	setsid();
	close(0);
	close(1);
	close(2);
    }
}

/* ----------------------------------------------------------
 * FUNCTION	: copy_argv
 * DESCRIPTION	: This function will flatten argv into a
 *		: single string.  This function was taken
 *		: from the tcpdump source code.  Hopefully
 *		: someday I will get around to rewriting it.
 * INPUT	: 0 - argv
 * ---------------------------------------------------------- */
char *copy_argv(register char **argv)
{
    register char **p;
    register u_int len = 0;
    char *buf;
    char *src, *dst;

    p = argv;
    if (*p == 0)
	return 0;

    while (*p)
	len += strlen(*p++) + 1;

    buf = (char *)malloc(len);
    if (buf == NULL) {
	printf("[!] ERROR:  copy_argv: malloc\n");
	exit(1);
    }

    p = argv;
    dst = buf;

    while ((src = *p++) != NULL) {
	while ((*dst++ = *src++) != '\0')
	    ;
	dst[-1] = ' ';
    }
    dst[-1] = '\0';

    return buf;
}

int pcre_copy_substring(const char *subject, int *ovector, int stringcount,
	int stringnumber, char *buffer, int size)
{
    int yield;
    if (stringnumber < 0 || stringnumber >= stringcount)
	  return -7;  /* PCRE_ERROR_NOSUBSTRING */
    stringnumber *= 2;
    yield = ovector[stringnumber+1] - ovector[stringnumber];
    if (size < yield + 1) return -6;  /* PCRE_ERROR_NOMEMORY */
    memcpy(buffer, subject + ovector[stringnumber], yield);
    buffer[yield] = 0;
    return yield;
}

